name = 'sour-cereal'
version = '1.0.8'

author = 'Carlos Alberto Duarte Pinto'
author_email = 'carlos.adpinto@gmail.com'

keywords = ['abstraction', 'data extraction', 'data sources', 'interface',
            'etl']

description = 'An abstraction of data source for extraction applications usage'

url = 'https://github.com/CarlosAdp/sour-cereal'

download_url = 'https://github.com/CarlosAdp/sour-cereal/archive/' +\
        f'v{version}.tar.gz'

classifiers = [
    'Programming Language :: Python :: 3.6',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Operating System :: OS Independent',
    'Topic :: Utilities',
    'License :: OSI Approved :: MIT License',
    'Intended Audience :: Developers'
]

license = 'MIT'



from .base.extraction import Extraction     # noqa
from .source_connection import SourceConnection     # noqa
from .auth_source_connection import AuthSourceConnection    # noqa
from .session_source_connection import SessionSourceConnection  # noqa
from .__about__ import version


__version__ = version



'''Defines an extraction.'''
from typing import Any, List, Tuple
from datetime import datetime

from .source_connection_interface import SourceConnectionInterface


class Extraction:
    '''Describe the procedure of data fetching.

    :param source: a data source class or instance
    :type source: SourceConnection
    :param parameters: a dictionary with the extraction parameters
    :type parameters: dict, optional

    An extraction describes the procedure of data fetching from a source. It's
    instantiation requires a data source and a dictionary of parameters.

    Every extraction may have a fingerprint and a list of pairs (timestamp,
    status), from which one may check if the extraction is ready or not.
    '''
    source: SourceConnectionInterface
    parameters: Any
    fingerprint: Any
    status_over_time: List[Tuple[datetime, Any]]

    def __init__(
        self: 'Extraction',
        source: SourceConnectionInterface,
        parameters: Any = None
    ) -> None:
        '''Please, refer to the class documentation.'''
        self.source = source
        self.parameters = parameters
        self.fingerprint = None
        self.status_over_time = []

    def prepare(self: 'Extraction') -> None:
        '''Prepare the extraction and set its fingerprint.

        :raises IOError: raised when the method does not succeed.
        '''
        self.fingerprint = self.source.prepare_extraction(self.parameters)

    def get_all_status(self: 'Extraction') -> List[Tuple[datetime, Any]]:
        '''Return all extraction's status

        :return: the description of the current status
        :rtype: List[Tuple[datetime, Any]]
        '''
        return self.status_over_time

    def get_current_status(self: 'Extraction') -> Any:
        '''Return only the current status

        :return: only the current status of the extraction
        :rtype: Any
        '''
        if len(self.status_over_time) > 0:
            return self.status_over_time[-1][1]

        return None

    def update_status(self: 'Extraction') -> None:
        '''Update status list with a new status.

        :raises IOError: raised when the method does not succeed.
        '''
        self.status_over_time.append((
            datetime.now(),
            self.source.get_status_of_extraction(self.fingerprint)
        ))

    def is_ready(self: 'Extraction') -> bool:
        '''Indicate whether the extraction is ready or not (may use the
        `get_current_status` method).

        :raises IOError: raised when the method does not succeed.
        :return: `True` if it's ready and `False` otherwise
        :rtype: bool
        '''
        return self.source.check_availability_of_extraction(
            self.fingerprint,
            self.get_current_status()
        )

    def is_done(self: 'Extraction') -> bool:
        '''Indicate whether the extraction is done/finished/executed (may use
        the `get_current_status` method).

        :raises IOError: raised when the method does not succeed.
        :return: `True` if it's done/finished/executed and does not need fur-
        ther execution. `False` otherwise
        :rtype: bool
        '''
        return self.source.check_availability_of_extraction(
            self.fingerprint,
            self.get_current_status()
        )

    def execute(self: 'Extraction') -> Any:
        '''Execute the extraction and return a result.

        :raises IOError: raised when the method does not succeed.
        :return: the result of the extraction. It may be a list of filenames to
        which data was writing, the data itself, etc.
        :rtype: Any
        '''
        if self.fingerprint is None:
            self.prepare()

        return self.source.execute_extraction(
            self.fingerprint,
            self.get_current_status()
        )

    def clean_resources(self: 'Extraction') -> bool:
        '''Attempt to clean resources that might have been allocated for the
        extraction and return this operation's success (or failure).

        :raises IOError: raised when the method does not succeed.
        :return: `True` if the resources got cleaned and `False` otherwise
        :rtype: bool
        '''
        return self.source.clean_resources_of_extraction(self.fingerprint)



from typing import Any

from .source_connection import SourceConnection


class AuthSourceConnection(SourceConnection):
    '''Auth-based source connection.

    :param credentials: user or system credentials used for the authentication
    :type credentials: Any
    '''

    def __init__(self: 'AuthSourceConnection', credentials: Any) -> None:
        '''Please refer to this class documentation.'''



from typing import Any

from .base.source_connection_interface import SourceConnectionInterface
from .base.extraction import Extraction


class SourceConnection(SourceConnectionInterface):
    '''Abstract class for a basic data source connection.

    It implements a base interface and adds a new method for creating extrac-
    tions.

    This class was written alongside and using a base interface for modularity
    purposes.
    '''
    def new_extraction(
        self: 'SourceConnection',
        parameters: Any = None
    ) -> Extraction:
        '''Return a new extraction with parameters and this instance as source.
        '''
        return Extraction(source=self, parameters=parameters)

    def get_prepared_extraction(
        self: 'SourceConnection',
        fingerprint: Any
    ) -> Extraction:
        '''Return a prepared extraction according to a given fingerprint.

        :param fingerprint: an existing extraction's fingerprint
        :type fingerprint: Any
        :return: an extraction defined by the given fingerprint
        :rtype: Extraction
        '''
        extraction = Extraction(source=self)
        extraction.fingerprint = fingerprint

        return extraction



'''Defines abstraction for any data source.'''
from abc import ABC, abstractmethod
from typing import Any


class SourceConnectionInterface(ABC):
    '''Interface for any kind of data source, comprising it's minimal set of
    methods.
    '''

    def prepare_extraction(
        self: 'SourceConnectionInterface',
        parameters: Any = None,
    ) -> dict:
        '''Prepare an extraction and return a fingerprint for further queries.

        This method shall raise an IOError if it does not succeed.

        :param parameters: a dictionary with the extraction parameters
        :type parameters: dict, optional
        :raises IOError: raised when the method does not succeed.
        :return: the extraction's fingerprint, used to getting status, execu-
        ting and cleaning
        :rtype: Any
        '''
        return parameters

    def get_status_of_extraction(
        self: 'SourceConnectionInterface',
        extraction_fingerprint: Any = None,
    ) -> Any:
        '''Return the current status of a prepared extraction using its finger-
        print.

        This method shall raise an IOError if it does not succeed.

        :param extraction_fingerprint: the ethe extraction's identifier
        :type extraction_fingerprint: Any, optional
        :raises IOError: raised when the method does not succeed.
        :return: the description of the current status of an extraction
        :rtype: Any
        '''
        return None

    def check_availability_of_extraction(
        self: 'SourceConnectionInterface',
        extraction_fingerprint: Any = None,
        status: Any = None,
    ) -> bool:
        '''Indicate whether an extraction is ready or not according to a speci-
        fic status.

        :param extraction_fingerprint: the ethe extraction's identifier
        :type extraction_fingerprint: Any, optional
        :param status: a specific status of the extraction
        :type status: Any, optional
        :return: `True` if it's ready and `False` otherwise
        :rtype: bool
        '''
        return True

    def check_completion_of_extraction(
        self: 'SourceConnectionInterface',
        extraction_fingerprint: Any = None,
        status: Any = None,
    ) -> bool:
        '''Indicate whether an extraction is finished (executed) or not.

        :param extraction_fingerprint: the ethe extraction's identifier
        :type extraction_fingerprint: Any, optional
        :param status: a specific status of the extraction
        :type status: Any, optional
        :return: `True` if the extraction is done/finished/complete and does
        not need further execution. `False` otherwise
        :rtype: bool
        '''
        return True

    @abstractmethod
    def execute_extraction(
        self: 'SourceConnectionInterface',
        extraction_fingerprint: Any = None,
        status: Any = None,
    ) -> Any:
        '''Execute an extraction and returns a result.

        This method shall raise an IOError if it does not succeed.

        :param extraction_fingerprint: the ethe extraction's identifier
        :type extraction_fingerprint: Any, optional
        :param status: a specific status of the extraction
        :type status: Any, optional
        :raises IOError: raised when the method does not succeed.
        :return: the result of the extraction. It may be a list of filenames to
        which data was writing, the data itself, etc.
        :rtype: Any
        '''
        pass

    def clean_resources_of_extraction(
        self: 'SourceConnectionInterface',
        extraction_fingerprint: Any = None,
    ) -> bool:
        '''Attempt to clean resources that might have been allocated for an ex-
        traction and return this operation's success (or failure).

        This method shall raise an IOError if it does not succeed.

        :param extraction_fingerprint: the ethe extraction's identifier
        :type extraction_fingerprint: Any, optional
        :raises IOError: raised when the method does not succeed.
        :return: `True` if the resources got cleaned and `False` otherwise
        :rtype: bool
        '''
        return True



import setuptools


about = {}
with open('sour_cereal/__about__.py') as fd:
    exec(fd.read(), about)


with open('README.md', 'r') as fd:
    long_description = fd.read()


setuptools.setup(
    name=about['name'],
    version=about['version'],
    author=about['author'],
    author_email=about['author_email'],
    description=about['description'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=about['keywords'],
    url=about['url'],
    download_url=about['download_url'],
    packages=setuptools.find_packages(),
    install_requires=[],
    classifiers=about['classifiers'],
    license=about['license']
)



from .auth_source_connection import AuthSourceConnection


class SessionSourceConnection(AuthSourceConnection):
    '''A connection to a session-based data source.

    A session will not be opened at the moment of instantiation, but using the
    `open` method instead.
    '''

    def open(self: 'SessionSourceConnection') -> bool:
        '''Opens a session attached to this connection.'''
        return True

    def close(self: 'SessionSourceConnection') -> bool:
        '''Closes the attached session'''
        return True
